/**********************************************************************
 *
 * gsdltools.cpp -- 
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "gsdltools.h"

#if defined(__WIN32__)
#include <windows.h>
#include <process.h>
#endif

bool littleEndian() {
  char s[2] = {'\xFE', '\xEF'};

  if (sizeof(unsigned short) == 2)
    return *(unsigned short*)s == 0xEFFE;
  else if (sizeof(unsigned int) == 2)
    return *(unsigned int*)s == 0xEFFE;
}

text_t dm_safe (const text_t &instring) {

  text_t outstring;
  text_t::const_iterator here = instring.begin();
  text_t::const_iterator end = instring.end();
  while (here != end) {
    if (*here == '_' || *here == '\\') outstring.push_back('\\');
    outstring.push_back(*here);
    here ++;
  }
  return outstring;
}

// gsdl_system spawns a completely separate program (i.e. the calling
// program continues and terminates normally). Arguments containing special
// characters (e.g. '&') should be quoted with ""

#if defined (__WIN32__)
void gsdl_system (char *cmd, ostream &logout) {

  STARTUPINFO ps = {sizeof(STARTUPINFO), NULL, NULL, NULL,
                    0, 0, 0, 0, 0, 0,
                    0, 0,
                    0, 0, NULL,
                    NULL, NULL, NULL};
  PROCESS_INFORMATION pi;
  BOOL res = CreateProcess(NULL,
                           cmd,
                           NULL,
                           NULL,
                           FALSE,
                           DETACHED_PROCESS,
                           NULL,
                           NULL,
                           &ps,
                           &pi);
  if (!res) {
    logout << "Failed to start " << cmd << " process, error code " << GetLastError();
  } 

  CloseHandle(pi.hProcess);
  CloseHandle(pi.hThread);
}

#else
void gsdl_system (char *cmd, ostream &logout) {

  if (cmd == NULL) return;
  int pid = fork();
  if (pid == -1) return;
  if (pid == 0) {
    // child process
    char *argv[4];
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = cmd;
    argv[3] = 0;
    execv("/bin/sh", argv);
    //    exit(127);
  }
}
#endif


// gsdl_call_perl executes the perl script perl_cmd, passing it all the 
// arguments provided. 

// Arguments should be char*'s, the last argument should be NULL

// perl_cmd shouldn't contain the path to the script as we use "perl -S"
// to find it. This means that the PATH environment variable of the calling
// process should contain the directory where the perl_cmd script lives.

// Uses a standard system() call on unix and a synchronous _spawn() on windows.
// We use the _spawn() because windows 9* doesn't seem to pass the 
// environment of the calling process to the new process when using
// system (which we need to do).

// all arguments will be quoted with double quotes [""] so they should be unquoted
// when passed in.

// returns the exit status of the called process (-1 if system() or _spawn() failed)

int gsdl_call_perl (char *perl_cmd, ...) {

  va_list argn;
  char *thisarg = perl_cmd;

#if defined(__WIN32__) && !defined (__GNUC__)
  int i = 2;
#define GSDL_MAX_ARGS 20
  char *args[GSDL_MAX_ARGS];
  args[0] = "perl";
  args[1] = "-S";
#else
  text_t cmd = "perl -S";
#endif

  // get arguments
  va_start(argn, perl_cmd);
  while(thisarg != NULL) {
#if defined(__WIN32__) && !defined (__GNUC__)
    if (i >= GSDL_MAX_ARGS) break;
    text_t tmp = thisarg;
    tmp = "\"" + tmp + "\"";
    args[i] = tmp.getcstr();
    i++;
#else
    cmd += " \"";
    cmd += thisarg;
    cmd.push_back ('"');
#endif
    thisarg = va_arg(argn, char *);
  }
  va_end(argn); 

#if defined(__WIN32__) && !defined (__GNUC__)
  args[i] = NULL;
  int rv = _spawnvp (_P_WAIT, "perl", args);
  while (--i > 2) delete (args[i]);
#else
  char *cmdc = cmd.getcstr();
  int rv = system (cmdc);
  delete (cmdc);
#endif
  return rv;
}

// attempts to work out if perl is functional
bool perl_ok () {
#if defined(__WIN32__)
  int i = gsdl_call_perl ("-e", "exit 1", NULL);
  return (i == 1);
#else
  int i = gsdl_call_perl ("-e", "'exit 0'", NULL);  
  return (i == 0);
#endif
}
